# frozen_string_literal: true

require 'thor'
require 'httparty'

 
## 
# Development mode only
#
if ENV['RUBY_ENV'] == 'development'
  require 'byebug'
  require 'awesome_print'
end
#
#
require_relative "weighflow_cli/version"
require_relative 'weighflow_cli/rendering'
require_relative 'weighflow_cli/client'
require_relative 'weighflow_cli/cli'
require_relative 'weighflow_cli/credentials'
require_relative 'weighflow_cli/order_handler'

 

module WeighflowCli 

  class Error < StandardError; end
  # 
  ROOT_PATH = File.expand_path('~/.weighflow_cli').freeze
  #
  Dir.mkdir(ROOT_PATH) unless Dir.exists?(ROOT_PATH)
  #
   
  # Provide an override to relocate data directory
  def self.data_path     
    return @data_path if defined?(@data_path)
    @data_path ||= ENV.fetch('WEIGHFLOW_CLI_PATH') { File.join(WeighflowCli::ROOT_PATH, 'data') }
    ensure_data_path
    @data_path
  end

  ##
  # Ensure data path
  def self.ensure_data_path
    Dir.mkdir(data_path) unless Dir.exists?(data_path)
  end
   #
  
   Configuration = Struct.new(:secret, :url)

   ##
   # This can be used to set the +secret+ and +url+ for api credentials
   # this setting can be done at the console, via
   #   weighflow_cli login
   # or through in code if using the gem within your own code by supplying
   # a block and passing in the credientials.
   #
   def self.configuration
    if block_given?
      configuration = Configuration.new
      yield(configuration)      
      @config = configuration.to_h
    end      
    @config ||= Credentials.load_config!    
   end
 
  
   ##
   # Creates an instance of the API client lib for direct access to the api
  def self.client    
    @client ||= Client.new(configuration)  
  end
 

  ## 
  # Pull down the orders and store in the local repository data path
  # * This is used to management order changes
  # 
  def self.pull
    WeighflowCli::OrderHandler.pull
  end


  ##
  # Find a specific order based on the +external_id+ given withing the orders
  # + Helps to pull order details when needed.
  def self.find_order(external_id)
    WeighflowCli::OrderHandler.find_order(external_id)
  end


  def self.list(indexes_only: false)
    WeighflowCli::OrderHandler.list(indexes_only: indexes_only)
  end

  def self.weights
    client.weights
  end


  def self.create_weight(params: {})
    client.create_weight(params: params)
  end
  
  def self.void_weight(weight_id, reason)
    client.void_weight(weight_id, reason)
  end
 
end
