require 'digest'

 

module WeighflowCli

  module OrderHandler
    #
    # Pull order data from Weighflow and store in local repository
    def self.pull
      ##
      # api methods to collect orders
      method_names = [:commodity_outbound_truck, 
                      :commodity_inbound_truck,
                      :bulk_inventory_outbound_truck, 
                      :bulk_inventory_inbound_truck]
      #
      # collect orders using threads
      results = threaded_api_query(method_names)
      #
      # flatten order array of hashes       
      orders = results.map { |orders| orders[:orders] }.flatten 
      #
      # Store in repository
      diff = OrderRepository.new(orders)      
      diff.perform      
    end


    ##
    # Process multiple api calls in threads
    def self.threaded_api_query(method_names)
      method_names.map do |name|
        Thread.new { WeighflowCli.client.send(name) }
      end.map(&:value)
    end

 
    #
    #
    # Find a single order with details from the stored repository
    def self.find_order(external_id)
      finder = Finder.new
      finder.find_order(external_id)
    end
    #
    #
    #


    #
    # List all orders currently in stored repository
    def self.list(indexes_only: false)
      result = []
      finder = Finder.new
      finder.for_each_index do |index|             
        if indexes_only
          result << index.index
        else
          result << index.data
        end
      end
      result
    end



    #
    #
    #
    # Helpers
    #
    module Handlers

      #
      # Data index file name
      def index_file_name 
        @index_file_name ||= File.join(WeighflowCli.data_path, 'orders_index_cache')
      end
        

      # 
      # MD5 hash of order detail
      def create_checksum(order)
        Digest::MD5.hexdigest(order.to_json)
      end
      
      # 
      # Create the index simple line entry, for speed and 
      # change checking by recording an MD5 hash of the contents
      def index_line(order, file_name)
        order[:external_unique_id] + "|#{create_checksum(order)}|#{file_name}"
      end


      #
      # Build the file name from the checksum
      def build_detail_file_name(order)
        File.join(WeighflowCli.data_path, 'order_' + create_checksum(order) + '.json')
      end
 

      # 
      # Store all orders in data path
      #  - Saves the entire contents in separate file
      #  - Records the index of the filename with a checksum for detail reference
      def store_in_cache(orders) 
        File.open(index_file_name, 'w') do |index_file|
          # process each order
          orders.each do |order|
            detail_file_name = build_detail_file_name(order)          
            File.write(detail_file_name, JSON.pretty_generate(order))
            index_file.puts(index_line(order, detail_file_name))
          end 
        end
      end
 
      #
      # Loop through all indexes. This is memory effecient.  Does not load all into memory
      def for_each_index(&block)
        return unless File.exists?(index_file_name)
        File.foreach(index_file_name) do |li|
          compare_index, checksum, stored_json_file_name = li.chomp!.split('|')
          block.call OrderIndexResult.new(index: compare_index, checksum: checksum, file_name: stored_json_file_name)
        end
      end

      #
      # Find a matching index      
      def find_index(index)      
        for_each_index do |compare_index|
           return compare_index if compare_index.index == index
        end
      end

      #
      # Pluck the external unique id from the orders
      def external_ids(orders)
        orders.map { |order| order[:external_unique_id].to_s }
      end


      #
      # return orders that have been cached, but are no longer needed
      def find_deletes(orders)
        unique_ids = external_ids(orders)
        deletes = []
        for_each_index do |index_result|
          unless unique_ids.include?(index_result.index)
            deletes << index_result.data
            index_result.delete!
          end
        end
        deletes
      end


      #
      #
      #
      #
      class OrderIndexResult
        attr_reader :index, :checksum, :file_name

        def initialize(index:, checksum:, file_name:)
          @index = index
          @checksum = checksum
          @file_name = file_name
        end


        def data
          return @data if defined?(@data)
          return {} unless @file_name       
          return {} unless File.exists?(@file_name)
          @data = JSON.parse(File.read(@file_name), symbolize_names: true)
        end

        def delete! 
          File.delete(@file_name) if File.exists?(@file_name)
        end
      end
    end


    #
    # 
    class Finder
      include Handlers

      def find_order(external_id)
        find_index(external_id)
      end
    end



    class OrderRepository
      include Handlers

      attr_reader :orders

      def initialize(orders = [])
        @orders = orders 
      end

      def perform                
        adds = []
        updates = []
        unchanged = 0
        deletes = []

        # create file if not existing
        File.open(index_file_name, "w") {} unless File.exists?(index_file_name)
      
        #
        orders.each do |order|
          order_checksum = create_checksum(order)       
          index_result = find_index(order[:external_unique_id].to_s)
          
          if index_result 
            if index_result.checksum != order_checksum 
              index_result.delete!  # remove old, schedule update
              updates << order # setup update to be added to cache 
            else
              unchanged += 1           
            end                
          else
            adds << order
          end
        end

        # Remove and orders that will be deleted on store_in_cache
        deletes = find_deletes(orders) 
        #
        # Store all orders in data path with indexed file
        store_in_cache(orders)
        #
        #
        #
        { statistics: { adds: adds.size, updates: updates.size, deletes: deletes.size, unchanged: unchanged },
          adds: external_ids(adds),
          updates: external_ids(updates),
          deletes: external_ids(deletes)
        }
      end
    end
  end
end